#!/bin/sh
set -u

AGENT=""
DEBUG=""
HEADLESS=""
LAUNCHAGENT=""
REMOVE=""
TEMPAGENT=""
VERBOSE=""
VMNAME=""

VERSION="1.0.0"
VBOXMANAGE="/Applications/VirtualBox.app/Contents/MacOS/VBoxManage"

my_usage()
{
	echo ""
	echo "VBoxLaunchAgent [options] \"<VM_name>\""
	echo ""
	echo "options         [-h|--headless]"
	echo "                [-d|--debug]"
	echo "                [-r|--remove]"
	echo "                [-v|--verbose]"
	exit $1
}

showMessage()
{
	if [ "x"$VERBOSE = "x""true" ]; then
		echo "$@"
	fi
}

get_options()
{
	if [ $# -eq "0" ]; then
		echo "*** ERROR: No arguments specified. You must specify a VM."
		my_usage 1
	fi

	while test $# -ge 2;
	do
		ARG="$1"

		case "$ARG" in
		-d|--debug)
			DEBUG="true"
			shift
			;;

		-h|--headless)
			HEADLESS="true"
			shift
			;;

		-r|--remove)
			REMOVE="true"
			shift
			;;

		-v|--verbose)
			VERBOSE="true"
			shift
			;;

		*)
			echo "*** ERROR: Invalid argument: $ARG"
			my_usage 1
			;;
		esac
	done

	VMNAME="$1"
	AGENT=org.virtualbox.launch."$VMNAME"
	TEMPAGENT=/tmp/"$AGENT"
	LAUNCHAGENT=~/Library/LaunchAgents/"$AGENT".plist
	if [ "x"$VERBOSE = "x""true" ]; then
		echo ""
		echo "VERBOSE OUTPUT"
		echo "--------------------------------------------------------------------------------"
		echo "DEBUG        = $DEBUG"
		echo "HEADLESS     = $HEADLESS"
		echo "REMOVE       = $REMOVE"
		echo "VERBOSE      = $VERBOSE"
		echo "VMNAME       = $VMNAME"
		echo "AGENT        = $AGENT"
		echo "LAUNCHAGENT  = $LAUNCHAGENT"
		echo "TEMPAGENT    = $TEMPAGENT"
		echo ""
	fi
}

checkVBoxInstalled()
{
	if [ ! -e "$VBOXMANAGE" ]; then
		echo "VirtualBox is not installed."
		my_usage 1
	fi
}

vmExists()
{
	"$VBOXMANAGE" showvminfo "$VMNAME" >/dev/null 2>&1
	rc=$?
	if [ $rc != 0 ]; then
		echo "VM not found: $VMNAME"
		my_usage 1
	fi
	showMessage "VirtualBox VM found. . . : $VMNAME"
}

agentExists()
{
	if [ -e "$LAUNCHAGENT" ]; then
		echo "The Launch Agent ($LAUNCHAGENT) already exists!"
		echo "Do you want to overwrite it? (Yes/No)"
		printf "\a"

		read MY_ANSWER
		if [ "$MY_ANSWER" != "Yes" -a "$MY_ANSWER" != "YES" -a "$MY_ANSWER" != "yes" -a "$MY_ANSWER" != "Y" -a "$MY_ANSWER" != "y" ] ; then
			echo "Aborting Launch Agent creation. Your answer was: '$MY_ANSWER'".
			exit 2;
		fi
	fi
}

removeAgent()
{
	if [ "x"$REMOVE = "x""true" ]; then
		if [ -e "$LAUNCHAGENT" ]; then
			launchctl unload -F "$LAUNCHAGENT" >/dev/null 2>&1
			rm -f "$LAUNCHAGENT"
			showMessage "Removed Launch Agent . . : $LAUNCHAGENT"
		else
			showMessage "Launch Agent not found . : $LAUNCHAGENT"	
		fi
		exit 0
	fi
}

makeAgent()
{
	(
		echo "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
		echo "<!DOCTYPE plist PUBLIC \"-//Apple//DTD PLIST 1.0//EN\" \"http://www.apple.com/DTDs/PropertyList-1.0.dtd\">"
		echo "<plist version=\"1.0\">"
		echo "<dict>"
		echo "	<key>Label</key>"
		echo "	<string>org.virtualbox.launch.$VMNAME</string>"

		if [ "x"$DEBUG = "x""true" ]; then
		(
			echo "	<key>StandardOutPath</key>"
			echo "	<string>/tmp/$VMNAME.log</string>"
			echo "	<key>StandardErrorPath</key>"
			echo "	<string>/tmp/$VMNAME.error.log</string>"
		)
		fi

		echo "	<key>ProgramArguments</key>"
		echo "	<array>"
		echo "		<string>$VBOXMANAGE</string>"
		echo "		<string>startvm</string>"
		echo "		<string>$VMNAME</string>"

		if [ "x"$HEADLESS = "x""true" ]; then
			echo "		<string>--type</string>"
			echo "		<string>headless</string>"
		fi

		echo "	</array>"
		echo "	<key>RunAtLoad</key>"
		echo "	<true/>"
		echo "</dict>"
		echo "</plist>"
	) > "$TEMPAGENT"
	showMessage "Created temporary file . : $TEMPAGENT"
}

echo "================================================================================"
echo "$(basename $0) - Launch Agent creation tool for starting a VM at login"
echo "================================================================================"
echo "Version: $VERSION"
echo "Copyright (C) 2019, granada29 @ VirtualBox forums."
echo "with help from socratis @ VirtualBox forums."
echo "All rights reserved."
echo ""

checkVBoxInstalled
get_options "$@"
removeAgent
vmExists
agentExists
makeAgent

mv "$TEMPAGENT" "$LAUNCHAGENT"
echo "Created Launch Agent . . : $LAUNCHAGENT"

exit 0
